#include <vector>
#include <iostream>
#include <fstream>
#include "itoa.h"
#include "foundation_AMOS.hh"
#include <unistd.h>

using namespace std;
using namespace AMOS;

string OPT_BankName;
string OPT_LinkName;

void PrintUsage(const char* s) {

  cerr
    << "\nUSAGE: " << s << "  [-b bank] [-l links]\n\n";
  return;
}

void PrintHelp(const char* s) {

  PrintUsage (s);
  cerr
    << "-b path       Set path of link bank\n"
    << "-l path       Set path of links file\n"
    << "-h            Display help information\n"
    << endl;

  cerr
    << "Generates contig links from links file information\n"
    << endl;

  return;
}

void ParseArgs(int argc, char** argv) {

  int ch, errflg = 0;
  optarg = NULL;
  
  while(!errflg  &&
	((ch = getopt(argc, argv, "b:l:h")) != EOF))
    switch (ch) {

    case 'b':
      OPT_BankName = optarg;
      break;

    case 'l':
      OPT_LinkName = optarg;
      break;
      
    case 'h':
      PrintHelp(argv[0]);
	exit(EXIT_SUCCESS);
	break;
	
    default:
      errflg ++;
    }

  if(errflg > 0 || optind != argc) {

    PrintUsage(argv[0]);
    cerr << "Try '" << argv[0] << " -h' for more information.\n";
    exit(EXIT_FAILURE);
  }
}

int main(int argc, char ** argv) {

  ParseArgs (argc, argv);

  BankStream_t links(ContigLink_t::NCODE);
  links.create(OPT_BankName);

  BankStream_t contigs(Contig_t::NCODE);
  contigs.open(OPT_BankName);

  int link_id = 0;

  string read; string contig1; string or1; string contig2; string or2;
  ifstream link_data(OPT_LinkName.c_str());
  while(link_data >> read >> contig1 >> or1 >> contig2 >> or2) {

    ContigLink_t link; 
    link.setContigs(pair<ID_t, ID_t>(contigs.lookupIID(contig1), contigs.lookupIID(contig2)));
    link_id++; link.setIID(link_id); link.setEID(itoa(link_id)); 
    link.setSize(Size_t(0));
    
    link.setSD(SD_t(1));
    link.setType(ContigLink_t::OVERLAP);
    
    if(or1 == "EB" && or2 == "BE")
      link.setAdjacency(ContigLink_t::OUTIE);
    else if(or1 == "BE" && or2 == "EB")
      link.setAdjacency(ContigLink_t::INNIE);
    else if(or1 == "BE" && or2 == "BE")
      link.setAdjacency(ContigLink_t::NORMAL);
    else
	link.setAdjacency(ContigLink_t::ANTINORMAL);
    
    links << link;
  }
  
  links.close();

  return EXIT_SUCCESS;
}
