#include "foundation_AMOS.hh"
#include <map>
#include <string>
#include <iostream>

using namespace std;
using namespace AMOS;

void PrintUsage(const char* s) {

  cerr
    << "\nUSAGE: " << s << "  [bank-path]\n\n";
  return;
}

void PrintHelp(const char* s) {

  PrintUsage (s);

  cerr
    << "Removes duplicate reads from the bank\n"
    << endl;

  return;
}

#define DEBUG 0

Contig_t remove_duplicate_reads(Contig_t contig, map<ID_t, string>& IID2EID) {

  map<string, int> count;
  vector<Tile_t> new_tiling;

  for(vector<Tile_t>::iterator it = contig.getReadTiling().begin();
      it != contig.getReadTiling().end(); it++) {

    if(count[IID2EID[it->source]] == 0)
      new_tiling.push_back(*it);
    else
      cerr << "Found a duplicate read in " << contig.getEID() 
	   << " : " << IID2EID[it->source] << endl;

    count[IID2EID[it->source]]++;
  }

  contig.setReadTiling(new_tiling);
  return contig;
}

string chop_range(string EID) {

  if(EID[0] == 'E') {

    int start = EID.find_first_of('.');
    //int end = EID.find_last_of('.');
    //cout << EID << " " << start << " " << end;

    if(start > 0)
      //if(end > start) {
      //EID.erase(start, end-start); 
	//cout << " " << EID;
      //}
      //else {
	EID.erase(start, EID.length()-start); 
	//cout << " " << EID;
    //}

    //cout << endl;
  }

  return EID;
}

int main(int argc, char ** argv)
{
  if(argc != 2) 
    PrintHelp(argv[0]);

  char* OPT_BankPath = argv[1];

  map<ID_t, string> IID2EID;
  BankStream_t reads(Read_t::NCODE);
  reads.open(OPT_BankPath);

  Read_t read;
  while(reads >> read) 
    IID2EID[read.getIID()] = chop_range(read.getEID());

  reads.close();

  BankStream_t contigs(Contig_t::NCODE);
  contigs.open(OPT_BankPath);

  Contig_t contig; vector<Contig_t> new_contigs;
  while(contigs >> contig)
    new_contigs.push_back(remove_duplicate_reads(contig, IID2EID));
  contigs.close();

  contigs.create(OPT_BankPath);
  for(vector<Contig_t>::iterator it = new_contigs.begin(); it != new_contigs.end(); it++)
    contigs << *it;

  contigs.close();

  return EXIT_SUCCESS;
}
