#include <vector>
#include "itoa.h"
#include "foundation_AMOS.hh"
#include <iostream>
#include <stdio.h>
#include <unistd.h>

using namespace std;
using namespace AMOS;

string OPT_BankName;

void PrintUsage(const char* s) {

  cerr
    << "\nUSAGE: " << s << "  [-b bank] \n\n";
  return;
}

void PrintHelp(const char* s) {

  PrintUsage (s);
  cerr
    << "-b path       Set path of bank\n"
    << "-h            Display help information\n"
    << endl;

  cerr
    << "Merges adjacent contigs in a scaffold into a single contig.\n"
    << endl;

  return;
}

void ParseArgs(int argc, char** argv) {

  int ch, errflg = 0;
  optarg = NULL;
  
  while(!errflg  &&
	((ch = getopt(argc, argv, "b:h")) != EOF))
    switch (ch) {

    case 'b':
      OPT_BankName = optarg;
      break;
      
    case 'h':
      PrintHelp(argv[0]);
	exit(EXIT_SUCCESS);
	break;
	
    default:
      errflg ++;
    }

  if(errflg > 0 || optind != argc) {

    PrintUsage(argv[0]);
    cerr << "Try '" << argv[0] << " -h' for more information.\n";
    exit(EXIT_FAILURE);
  }
}


int main(int argc, char ** argv) {

  ParseArgs (argc, argv);

  BankStream_t scaffolds(Scaffold_t::NCODE);
  scaffolds.open(OPT_BankName);
  Scaffold_t scaff;
  int merged_contig_id = 1;

  BankStream_t contigs(Contig_t::NCODE);
  contigs.open(OPT_BankName);
  vector<Contig_t> new_contigs;

  while(scaffolds >> scaff) {

    if(scaff.getContigTiling().size() > 1) {

      Contig_t merged_contig; char id[10]; sprintf(id, "%05d", merged_contig_id++); 
      merged_contig.setIID(contigs.getMaxIID()+1);
      merged_contig.setEID(string("merged") + id);
      
      for(vector<Tile_t>::iterator it = scaff.getContigTiling().begin(); 
	  it != scaff.getContigTiling().end(); it++) {

	contigs.seekg(it->source);

	Contig_t current;
	contigs >> current; 

	if(it->range.isReverse()) 
	  current.reverseComplement();

	for(vector<Tile_t>::iterator it2 = current.getReadTiling().begin();
	    it2 != current.getReadTiling().end(); it2++) {

	  it2->offset += it->offset;
	  
	  merged_contig.getReadTiling().push_back(*it2);
	}

	merged_contig.setSequence(merged_contig.getSeqString()+current.getSeqString(),
				  merged_contig.getQualString()+current.getQualString());

	contigs.remove(it->source);
      }

      new_contigs.push_back(merged_contig);
    }
  }

  contigs.clean(); contigs.close();
  contigs.open(OPT_BankName);
  int max_id = contigs.getMaxIID()+1;

  for(int i = 0; i < new_contigs.size(); i++) {

    new_contigs[i].setIID(max_id+i);
    contigs << new_contigs[i];
  }

  contigs.close();
  scaffolds.close();

  return EXIT_SUCCESS;
}
